<?php
namespace plugins\file_manager\controllers;

use plugins\file_manager\library\Breadcrumbs;
use plugins\file_manager\library;
use plugins\file_manager\services\FileSystem;
use plugins\file_manager\library\exceptions;

include( dirname( dirname( __FILE__ ) ) . DS . 'library' . DS . 'base64url.php' );

/**
 * Directory controller
 * @author Jérôme Brilland
 * @version 2.1
 * @copyright (c) 2021-2022, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class DirectoryController extends \ActionController {

    use Breadcrumbs;

    private $filesystem;
    private $fullpath;

    protected function beforeFilter() {
        $authorization = \IoC::resolve( 'authorization' );
        $authorization->authorize( 'file_manager.manage_files' );
        $i18n = \IoC::resolve( 'i18n' );
        $this->title = $i18n->__( 'files' );
        $this->filesystem = new FileSystem( $this );
        $this->decodedPath = ( !empty( $this->request->getAttribute( 'path', '' ) ) ) ? library\base64url_decode( $this->request->getAttribute( 'path' ) ) : '';
        $this->fullpath = PUBLIC_DIR . $this->decodedPath;
        $this->encodedPath = $this->request->getAttribute( 'path', '' );
    }

    public function doIndex() {
        $i18n = \IoC::resolve( 'i18n' );
        
        try {
            $this->files = array_map(
                function( $file ) {
                    $file['encoded_path'] = library\base64url_encode( $this->decodedPath . DS . $file['name'] );
                    return $file;
                },
                $this->filesystem->readDirectory( $this->fullpath ));

            $this->breadcrumbs = $this->directoryToBreadcrumb( $this->decodedPath );
            if ( !is_writable( $this->fullpath ) ) {
                $this->flash['warning'] = $i18n->__( 'directory_not_writable' );
            }
        } catch ( exceptions\DirectoryNotReadableException $e ) {
            $this->flash['error'] = $i18n->__( 'directory_not_readable' );
            $this->redirect( BASE_URL . 'file_manager/' . library\base64url_encode( dirname( $this->decodedPath ) ) );
        } catch ( \RuntimeException $e ) {
            $this->redirect( BASE_URL . 'file_manager/' );
        }
    }

    public function doCreate() {
        $i18n = \IoC::resolve( 'i18n' );
        try {
            $directory = $this->request->getParameters()->getString( 'directory', null, 'unsafe_raw', FILTER_FLAG_STRIP_LOW );
            $this->filesystem->createDirectory( $this->fullpath . DS . $directory );           
        } catch ( exceptions\FileExistsException $e ) {
            $this->flash['error'] = $i18n->__( 'file_already_exists', $directory );
        } catch ( RuntimeException $e ) {
            $this->flash['error'] = $i18n->__( 'directory_not_created', $directory );
        } finally {
            $this->redirect( BASE_URL . 'file_manager/' . $this->encodedPath );
        }
    }

    public function doDelete() {
        $i18n = \IoC::resolve( 'i18n' );
        try {
            $this->filesystem->deleteDirectory( $this->fullpath );
        } catch ( Exception $e ) {
            $this->flash['error'] = $i18n->__( 'directory_not_deleted', basename( $this->decodedPath ) );
        } finally {
            $this->redirect( BASE_URL . 'file_manager/' . library\base64url_encode( dirname( $this->decodedPath ) ) );
        }
    }

}
